﻿using H_Graph;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Drawing;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Drawing.Imaging;
using Rectangle = System.Drawing.Rectangle;
using Color = System.Drawing.Color;

namespace Test_dll6
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        HydraulicGraph graphPaper;

        ElementGroup ListOfElements, TheGridLines;
        bool Metric = false;

        public MainWindow()
        {
            InitializeComponent();
            ListOfElements = new ElementGroup("Arial",10 , Metric);
            TheGridLines   = new ElementGroup("Arial", 10, Metric);

            ShowTheGraph();
        }
        private BitmapSource CreateBitmapSourceFromGdiBitmap(Bitmap bitmap)
        {
            if (bitmap == null)
                throw new ArgumentNullException("bitmap");

            var rect = new Rectangle(0, 0, bitmap.Width, bitmap.Height);

            var bitmapData = bitmap.LockBits(
                rect,
                ImageLockMode.ReadWrite,
                System.Drawing.Imaging.PixelFormat.Format32bppArgb);

            try
            {
                var size = (rect.Width * rect.Height) * 4;

                return BitmapSource.Create(
                    bitmap.Width,
                    bitmap.Height,
                    bitmap.HorizontalResolution,
                    bitmap.VerticalResolution,
                    PixelFormats.Bgra32,
                    null,
                    bitmapData.Scan0,
                    size,
                    bitmapData.Stride);
            }
            finally
            {
                bitmap.UnlockBits(bitmapData);
            }
        }

        private void CreateTheGraph()
        {
            // set current XML GraphCanvas size
            GraphCanvas.Width = Width - 130;
            GraphCanvas.Height = Height - 165;

            // create an internal graphic canvas to draw on
            Graphics _graphic = Graphics.FromHwnd(
                        new System.Windows.Interop.WindowInteropHelper(this).Handle);

            // create the graph paper class with default graph colors
            graphPaper = new HydraulicGraph((int)GraphCanvas.Width, (int)GraphCanvas.Height);

            // create the graph paper with custom colors
            // alternate to the default constructor
            //graphPaper = new HydraulicGraph((int)GraphCanvas.Width, (int)GraphCanvas.Height,
            // Color.LightGray,        // background 
            // Color.DarkBlue,         // lines
            // Color.Black,            // border
            // Color.Gray);            // intermediate lines

            // add some lines, points, and text to the list of elements
            AddExampleElements();

            // entering Zeros allow the graph to auto scale
            // other numbers will hard fix the graph limits
            // e.g. graphPaper.GetGraphLimits(ListOfElements, 2000, 150);
            (float MAX_Q, float MAX_P) = graphPaper.GetGraphLimits(ListOfElements, 0, 0);

            // create a list of elements to draw the grid lines etc.
            TheGridLines = graphPaper.CreateTheGrid(
                            _graphic,                   // graphic canvas
                            MAX_Q, MAX_P,               // graph limits
                            "Calibri", 10,    // axis font
                            true,                       // intermediate line flag
                            false,                      // SVG spacing of text
                            Metric);
        }

        private void ShowTheGraph()
        {
            CreateTheGraph();

            Bitmap bm = graphPaper.DrawBitmapGraph(ListOfElements, TheGridLines);
            theGraph.Source = CreateBitmapSourceFromGdiBitmap(bm);
        }

        private void AddExampleElements()
        {
            //add 3 lines
            ListOfElements.AddLine(Color.Black, 2, 0, 15, 450, 70, GraphicStyle.dotted, TAG.none, false);
            ListOfElements.AddLine(Color.Blue, 2, 0, 10, 800, 70, GraphicStyle.solid, TAG.none, true);
            ListOfElements.AddLine(Color.Red, 2, 0, 88, 800, 30, GraphicStyle.solid, TAG.none, true);

            //find flow point on line and plot
            float targetPressure = Plots.FindPonLine(0, 88, 800, 30, 500); // 500 = target flow
            ListOfElements.AddPoint(Color.Red, 500, targetPressure, GraphicStyle.circFilled, 4, TAG.none);

            // find presure point on line and plot
            float targetFlow = Plots.FindQonLine(0, 88, 800, 30, 20);  // 20  = target pressure
            ListOfElements.AddPoint(Color.Green, targetFlow, 20, GraphicStyle.rectangle, 5, TAG.none);

            // intersect two lines
            float Flow = 0, Press = 0;
            ElementGroup.Intersect2LinesWithHose(
                        0, 10, 800, 70,     // Line 1
                        0, 88, 800, 30,     // Line 1
                        0,                  // Hose
                        ref Flow, ref Press,
                        false);             // will restrict the answer to the end point of the lines
            // plot and label the point
            ListOfElements.AddPoint(Color.Red, Flow, Press, GraphicStyle.doublecirc, 5, TAG.none);
            ListOfElements.AddTextToPoint(Flow, Press, TAG.none);

            // Add some text to the graph
            ListOfElements.AddText(Color.Red, Color.White, true, "Added Text", "Calibri", 24, 600, 60, TAG.none);
        }

    }
}
